@@ -16,8 +16,8 @@
 #
 
 """databand bindings for Google Cloud Storage"""
-
-
+import base64
+import hashlib
 import logging
 import mimetypes
 import os
@@ -318,7 +318,7 @@ class GCSClient(FileSystem):
     def put_multiple(
         self, filepaths, remote_directory, mimetype=None, chunksize=None, num_process=1
     ):
-        if isinstance(filepaths, str):
+        if isinstance(filepaths, six.string_types):
             raise ValueError(
                 "filenames must be a list of strings. If you want to put a single file, "
                 "use the `put(self, filename, ...)` method"
@@ -452,18 +452,28 @@ class GCSClient(FileSystem):
             ):
                 yield it
 
-    def download(self, path, chunksize=None, chunk_callback=lambda _: False):
+    def _open_read(
+        self,
+        remote_path,
+        local_path=None,
+        delete_file_on_close=True,
+        chunksize=None,
+        chunk_callback=lambda _: False,
+    ):
         """Downloads the object contents to local file system.
 
         Optionally stops after the first chunk for which chunk_callback returns True.
         """
         chunksize = chunksize or self.chunksize
-        bucket, obj = self._path_to_bucket_and_key(path)
+        bucket, obj = self._path_to_bucket_and_key(remote_path)
 
-        tmp_file = get_local_tempfile(os.path.basename(path))
-        with open(tmp_file, "wb") as fp:
+        tmp_file_path = local_path or get_local_tempfile(os.path.basename(remote_path))
+        with open(tmp_file_path, "wb") as fp:
             # We can't return the tempfile reference because of a bug in python: http://bugs.python.org/issue18879
-            return_fp = _DeleteOnCloseFile(tmp_file, "r")
+            if delete_file_on_close:
+                return_fp = _DeleteOnCloseFile(tmp_file_path, "r")
+            else:
+                return_fp = fp
 
             # Special case empty files because chunk-based downloading doesn't work.
             result = self.client.objects().get(bucket=bucket, object=obj).execute()
@@ -499,11 +509,33 @@ class GCSClient(FileSystem):
 
         return return_fp
 
+    def download_file(self, path, location=None):
+        """
+        Download file to local filesystem
+        :param path: remote path
+        :param location: local path or none
+        :return:
+        """
+        return self._open_read(
+            remote_path=path, local_path=location, delete_file_on_close=False
+        )
+
     def mkdir_parent(self, path):
         pass
 
-    def copy_from_local(self, local_path, dest):
+    def copy_from_local_file(self, local_path, dest):
         self.put(local_path, dest)
 
     def open_read(self, path, mode="r"):
-        return self.download(path)
+        return self._open_read(path)
+
+    def local_md5(self, local_path):
+        with open(local_path, "r+") as f:
+            checksum = hashlib.md5(f.read().encode("utf-8"))
+            base64_md5 = base64.b64encode(checksum.digest()).decode("utf-8")
+        return base64_md5
+
+    def remote_md5(self, remote_path):
+        bucket, obj = self._path_to_bucket_and_key(remote_path)
+        result = self.client.objects().get(bucket=bucket, object=obj).execute()
+        return result["md5Hash"]
